/*
   Basic Gameboy Advance Programming Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to display the letter A to a green background in the
   gameboy advance.  I know it does not sound like much but we all must start somewhere.
   The true purpose of this demo is to show you know to draw pixels to the screen.
   This is done by sending color values to register 0x6000000 which is defined in the gba.h
   as the PrimaryBuffer.  First thing we need to do in the gba app is to initialize the graphics mode.
   This is done by setting register 0x4000000 which is defined as REG_DISPCNT to the bits that
   represent the graphic mode you want to use.  For mode 3 this will be 0x3 and BG2 is 0x400.
   After we initialize the graphic mode, in this demo I clear the screen to green by setting
   all pixels to RGB16(0, 31, 0).  This macro called RGB16 will convert the values you see here
   into a single value that represents that color.  We take that value and assign it to the
   primary buffer to draw a pixel.
*/


// Demo: Basic Gameboy Advance Programming.


#include"gba.h"


// Function Prototypes...
void DrawVerticalLine(char startX, char endX, char y, char size, unsigned short int color);
void DrawHorizontalLine(char startY, char endY, char x, char size, unsigned short int color);
void ClearScreen(int width, int height, unsigned short int color);
void SetPixel(int x, int y, int width, unsigned short int color);


int main()
{
	char x,y;  	

   // First we initialize the graphics mode we will be working under using a define I created
   // called InitGraphicMode().
   InitGraphicMode(MODE_3 | BG2_ENABLE);

   // First lets clear the screen...how about to green.  I created a function called
   // clear screen that will take a color and clear the entire screen to it.
   ClearScreen(240, 160, RGB16(0, 31, 0));

   // Next lets draw the letter A in the screen.  We'll do this by drawing a few lines
   // with two functions I created.  The first function we'll see will draw a horizontal
   // line givin a start y, end y, x pos, screen width, and a color value.  This function
   // is called DrawHorizontalLine().  DrawVerticalLine does the same thing but this one
   // will draw a vertical.

   // Next lets draw two lines, 1 for every edge of the letter A.
   DrawHorizontalLine(30, 50, 40, 240, RGB16(31, 31, 31));
   DrawHorizontalLine(30, 50, 65, 240, RGB16(31, 31, 31));
   
   // Lastly we draw two straight lines for the top of the A and the middle of it.
   DrawVerticalLine(40, 65, 30, 240, RGB16(31, 31, 31));
   DrawVerticalLine(40, 65, 40, 240, RGB16(31, 31, 31));
   
   // We will keep looping forever until the user turns off the gameboy advance.
   while(1){ }

   return 0;
}


void DrawVerticalLine(char startX, char endX, char y, char size, unsigned short int color)
{
   // We'll start at the position in startX and keep going alone a line to endX.  The line
   // will be draw at the y position.  To actually draw a line we set the pixel position
   // of the screen to the specified color.
   for(; startX < endX; startX++)
      {
         SetPixel(startX, y, size, color);
      }
}


void DrawHorizontalLine(char startY, char endY, char x, char size, unsigned short int color)
{
   // We'll start at the position in startY and keep going down a line to endY.  The line
   // will be draw at the x position.  To actually draw a line we set the pixel position
   // of the screen to the specified color.
   for(; startY < endY; startY++)
      {
         SetPixel(x, startY, size, color);
      }
}


void ClearScreen(int width, int height, unsigned short int color)
{
   int x, y;

   // Here we loop through the whole screen and set every pixel to the specified color.
   for(x = 0; x < width; x++)
	   {
		   for(y = 0; y < height; y++)
		      {
               SetPixel(x, y, width, color);
		      }
	   }
}


void SetPixel(int x, int y, int width, unsigned short int color)
{
   // SetPixel will pretty much just set a pixel value to a specified location.
   PrimaryBuffer[x + y * width] = color;
}


// Recap:

// Well it is not much but it is a start in a positive direction.  Pretty much all we are
// doing in this program is plotting pixels to form the letter A (for Ace).  I created
// two function to do this (one to draw a horizontal line and one for vertical lines) to make
// our lives a little bit easier but we it comes to displaying text we will not be using this.
// Now that we know how to plot pixels the rest of the demo will build off of this to do
// more advance things.


// Copyright September 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com
